/* Copyright (c) 2003 - 2015 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/
/***********************************************************************/
/*                                                                     */
/*  FlashOS.H:  Data structures and entries                            */
/*              for Flash Programming Functions                        */
/*                                                                     */
/***********************************************************************/

#ifndef __CYP15B102_H
#define __CYP15B102_H

#define CYP15B102_VERS       1           // Interface Version 1.01

#define CYP15B102_UNKNOWN    0           // Unknown
#define CYP15B102_ONCHIP     1           // On-chip Flash Memory
#define CYP15B102_EXT8BIT    2           // External Flash Device on 8-bit  Bus
#define CYP15B102_EXT16BIT   3           // External Flash Device on 16-bit Bus
#define CYP15B102_EXT32BIT   4           // External Flash Device on 32-bit Bus
#define CYP15B102_EXTSPI     5           // External Flash Device on SPI

#define CYP15B102_SECTOR_NUM 512         // Max Number of Sector Items
#define CYP15B102_PAGE_MAX   65536       // Max Page Size for Programming

#define CYP15B102_SZDEV      0x00020000  // Device Size in Bytes (128KB)
#define CYP15B102_SZPAGE     256         // Programming Page Size
#define CYP15B102_VALEMPTY   0x00        
//#define CYP15B102_USE_WIP 
//Some memory can be erased by going 0x000->all, others require page divisions
#define CYP15B102_USE_PAGES 
//Only TTFlash uses a sector erase
//#define CYP15B102_USE_SECTOR_ERASE  
#define CYP15B102_SECTOR_SIZE 0x001000

struct CYP15B102_FlashSectors  {
  unsigned long   szSector;    // Sector Size in Bytes
  unsigned long AddrSector;    // Address of Sector
};

#define CYP15B102_SECTOR_END 0xFFFFFFFF, 0xFFFFFFFF

//being position-independent means main code CANNOT ACCESS THESE FIELDS
struct CYP15B102_FlashDevice  {
   unsigned short     Vers;    // Version Number and Architecture
   char       DevName[128];    // Device Name and Description
   unsigned short  DevType;    // Device Type: ONCHIP, EXT8BIT, EXT16BIT, ...
   unsigned long    DevAdr;    // Default Device Start Address
   unsigned long     szDev;    // Total Size of Device
   unsigned long    szPage;    // Programming Page Size
   unsigned long       Res;    // Reserved for future Extension
   unsigned char  valEmpty;    // Content of Erased Memory

   unsigned long    toProg;    // Time Out of Program Page Function
   unsigned long   toErase;    // Time Out of Erase Sector Function

   struct CYP15B102_FlashSectors sectors[CYP15B102_SECTOR_NUM];
};

extern struct CYP15B102_FlashDevice const CYP15B102_FlashDevice;

#define CYP15B102_FLASH_DRV_VERS (0x0100+CYP15B102_VERS)   // Driver Version, do not modify!

/* 
 * Flash Programming Functions (Called by FlashOS)
 */

/*
 *  Initialize Flash Programming Functions
 *    Parameter:      adr:  Device Base Address
 *                    clk:  Clock Frequency (Hz)
 *                    fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */
extern int CYP15B102_Init(unsigned long adr, unsigned long clk, unsigned long fnc);

/*
 *  De-Initialize Flash Programming Functions
 *    Parameter:      fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */
extern int CYP15B102_UnInit(unsigned long fnc);  

extern int CYP15B102_BlankCheck(unsigned long adr, unsigned long sz, unsigned char pat);

/*
 *  Erase complete Flash Memory
 *    Return Value:   0 - OK,  1 - Failed
 */
extern int CYP15B102_EraseChip(void);

/*
 *  Erase Sector in Flash Memory
 *    Parameter:      adr:  Sector Address
 *    Return Value:   0 - OK,  1 - Failed
 */
extern int CYP15B102_EraseSector(unsigned long adr);

/*
 *  Program Page in Flash Memory
 *    Parameter:      adr:  Page Start Address
 *                    sz:   Page Size
 *                    buf:  Page Data
 *    Return Value:   0 - OK,  1 - Failed
 */
extern int CYP15B102_ProgramPage(unsigned long adr, unsigned long sz, unsigned char *buf);


extern unsigned long CYP15B102_Verify(unsigned long adr, unsigned long sz, unsigned char *buf);

#endif // __CYP15B102_H
