/***************************************************************************************
 * @file     xmodem.c
 * @version  V1.0
 * @date     12 September 2018
 *
 * @note
 * VORAGO Technologies
 *
 * @note
 * Copyright (c) 2013-2018 VORAGO Technologies.
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT.
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/
 
/*****************************************************************************/ 
/* Include files                                                             */ 
/*****************************************************************************/

#include <stdio.h>

#include "va108xx.h"
#include "bootloader.h"
#include "xmodem.h"
#include "uart.h"
#include "crc.h"
#include "board.h"
#include "main.h"

/*****************************************************************************/ 
/* Local pre-processor symbols/macros ('#define')                            */ 
/*****************************************************************************/

/** byte alignment macro */
#define ALIGNMENT(sz,align) (((sz)+((align)-1))&(~((align)-1)))

/** value to indicate packet verify failure */
#define XMODEM_FAIL_VALUE -1

/** value to indicate packet verify success */
#define XMODEM_PASS_VALUE 0

/** value in milliseconds between repeated sends of start transfer packet ('C') */
#define START_TRANSMISSION_REPEAT_MSEC 1000

/*****************************************************************************/ 
/* Local variable definitions ('static')                                     */ 
/*****************************************************************************/

// Packet storage. Double buffered version
static uint8_t m_au8RawPacket[2][ALIGNMENT(sizeof(stc_xmodem_packet_t),4)];

/*****************************************************************************/ 
/* Local function prototypes ('static')                                      */ 
/*****************************************************************************/

__INLINE static int32_t VerifyPacketChecksum(stc_xmodem_packet_t* pstcPacket, uint32_t u32SequenceNumber);

/*****************************************************************************/ 
/* Function implementation - global ('extern') and local ('static')          */ 
/*****************************************************************************/

/******************************************************************************
 * @brief Verifies checksum, packet numbering and
 * @param pkt The packet to verify
 * @param sequenceNumber The current sequence number.
 * @returns -1 on packet error, 0 otherwise
 *****************************************************************************/
__INLINE static int32_t VerifyPacketChecksum(stc_xmodem_packet_t* pstcPacket, uint32_t u32SequenceNumber)
{
  uint16_t u16PacketCRC;
  uint16_t u16CalculatedCRC;

  // Check the packet number integrity
  if (pstcPacket->u8PacketNumber + pstcPacket->u8PacketNumberC != 255)
  {
    return XMODEM_FAIL_VALUE;
  }

  // Check that the packet number matches the excpected packet number
  if (pstcPacket->u8PacketNumber != (u32SequenceNumber % 256))
  {
    return XMODEM_FAIL_VALUE;
  }
  
	// Calculate the CRC
  u16CalculatedCRC = CalculateCRC16((uint8_t *) pstcPacket->au8Data, (uint8_t *) &(pstcPacket->u8CrcHigh));
  u16PacketCRC = pstcPacket->u8CrcHigh << 8 | pstcPacket->u8CrcLow;

  // Check the CRC
  if (u16CalculatedCRC != u16PacketCRC)
  {
    return XMODEM_FAIL_VALUE;
  }
  return XMODEM_PASS_VALUE;
}

/******************************************************************************
 * @brief Starts a XMODEM download
 *
 * @param uint8_t* pu8BaseAddress
 *   The code RAM address to start writing to (usually the start of user app space)
 *
 * @param uint8_t* pu8EndAddress
 *   The last address - used to make sure we don't overrun our available space
 *   Usually the end of code RAM space, 0x20000
 *
 * @return size_t - the number of bytes received, or zero if there was an error
 *
 *****************************************************************************/
size_t XMODEM_ReceiveFile(uint8_t* pu8BaseAddress, uint8_t* pu8EndAddress)
{
  stc_xmodem_packet_t* pstcPacket;
  uint32_t u32Idx;
  uint32_t u32SequenceNumber = 1;	
	uint8_t* pu8CheckAddress;
	uint32_t u32Now;

  /* Send a start transmission packet. Wait for a response.
	   In PC_MODE, keep sending start packets until the transmission
     starts. In MCU<->MCU mode, send the start once.	*/
  while (1)
  {
    UART0TxByte(XMODEM_NCG);
		u32Now = u32MsCounter;
    while(u32MsCounter < (u32Now + START_TRANSMISSION_REPEAT_MSEC))
    {
			if (UART0IsRxByteAvailable())
      {
				// Goto is evil, but necessary to break from outer loop
        goto xmodem_transfer;
      }
			// feed watchdog
			WDFeed();
    }
#ifndef PC_MODE
		// in MCU<->MCU mode, only send the 'XMODEM_NCG' character once
		if (UART0IsRxByteAvailable()) { break; }
		else { return 0; }
#endif
  }
xmodem_transfer:
	// First, enable writing to code RAM space
	VOR_SYSCONFIG->ROM_PROT |= SYSCONFIG_ROM_PROT_WREN_Msk;
  while (1)
  {
    // Swap buffer for packet buffer.
    pstcPacket = (stc_xmodem_packet_t *)m_au8RawPacket[u32SequenceNumber & 1]; 

    // Fetch the first byte of the packet explicitly, as it defines the
    // rest of the packet.
    pstcPacket->u8Header = UART0RxByte();

    // Check for end of transfer.
    if (pstcPacket->u8Header == XMODEM_EOT)
    {
      // Acknowledge End of transfer.
      UART0TxByte(XMODEM_ACK);
      break;
    }

    // If the header is not a start of header (SOH), then cancel
    // the transfer.
    if (pstcPacket->u8Header != XMODEM_SOH)
    {
			VOR_SYSCONFIG->ROM_PROT &= ~SYSCONFIG_ROM_PROT_WREN_Msk; // disable writing to code RAM space
      return 0;
    }

    // Fill the remaining bytes packet.
    // Byte 0 is padding, byte 1 is header.
    for (u32Idx = 2; u32Idx < sizeof(stc_xmodem_packet_t); u32Idx++)
    {
      *(((uint8_t *) pstcPacket) + u32Idx) = UART0RxByte();
    }

    if (VerifyPacketChecksum(pstcPacket, u32SequenceNumber) != 0)
    {
      // On a malformed packet, we send a NAK, and start over.
      UART0TxByte(XMODEM_NAK);
      continue;
    }
		
		// Write data to code RAM
		pu8CheckAddress = pu8BaseAddress + ((u32SequenceNumber) * XMODEM_DATA_SIZE);
		if (pu8CheckAddress <= pu8EndAddress)
		{
		  memcpy((void*)(pu8BaseAddress + ((u32SequenceNumber - 1) * XMODEM_DATA_SIZE)),(const void*)pstcPacket->au8Data,XMODEM_DATA_SIZE);
		}

    u32SequenceNumber++;
    // Send ACK.
    UART0TxByte(XMODEM_ACK);
		
		// feed watchdog
	  WDFeed();
  }
	
	// We're done, so disable writing to code RAM space
	VOR_SYSCONFIG->ROM_PROT &= ~SYSCONFIG_ROM_PROT_WREN_Msk;
	return (u32SequenceNumber - 1) * XMODEM_DATA_SIZE;
}

/******************************************************************************
 * @brief Starts a XMODEM upload
 *
 * @param uint8_t* pu8BaseAddress
 *   The code RAM address to start sending from (usually the start of user app space)
 *
 * @param size_t szLength
 *   The length in bytes of the file to send
 *
 * @return size_t - the number of bytes sent, or zero if there was an error
 *
 *****************************************************************************/
size_t XMODEM_SendFile(uint8_t* pu8BaseAddress, size_t szLength)
{
	// TODO: implement
	return 0;
}

/*****************************************************************************/ 
/* End of file                                                               */ 
/*****************************************************************************/
